
#include <windows.h>
#include <stdio.h>
#include <conio.h>

#include "isdll.h"

//
// Global Variables
//
HANDLE	handle;		// Driver Handle
ULONG   venID=0;	// Vendor ID
ULONG   devID=0;	// Device ID

//
// Function Declarations
//
void		FindPciCard(void);
void		EnumeratePciCard(void);


//=********************** main ****************************
//=********************************************************
int   main(int argc, char *argv[])
{
	// Open UNINSYS Driver
	handle = UniDrv_Open(0);
	if( handle==0 )
	{
		printf("\nISPCI ERROR: Can't Open ISDRV\n\n" );
		_getch();
		return 1;
	}

	char* endptr;
	// Parse Command Line
	if( argc>1 )
		venID = strtoul(argv[1], &endptr, 16);
		//sscanf( argv[1], "%lX", &venID );
	if( argc>2 )
		devID = strtoul(argv[2], &endptr, 16);
		//sscanf( argv[2], "%lX", &devID );

	// Define PCI Cards
	if( venID!=0 && devID!=0 )
		FindPciCard();
	else
		EnumeratePciCard();

	UniDrv_Close(handle);
	_getch();
	return 0;
}

//=********************** FindPciCard *********************
//=********************************************************
void		FindPciCard(void)
{
	int         ret;
	PCI_DEVICE	pciDev;
	PCI_HEADER	pciHdr;
	ULONG       *ptr = (ULONG*)&pciHdr;


	ret = UniDrv_PCI_FindDevice( handle, (USHORT)devID, (USHORT)venID, 0, &pciDev );
	if( ret != 0)
	{
		printf("\nISPCI ERROR: Can't Find Device with VendorID=%X, DeviceID=%X\n\n", venID, devID );
		return;
	}
	ret = UniDrv_PCI_ReadCfgSpace( handle, &pciDev, &pciHdr );
	if( ret != 0)
	{
		printf("\nISPCI ERROR: Can't Read PCI Device Configuration Space\n\n" );
		return;
	}

	printf("Board: bus=0x%02X, dev=0x%02X\n", pciDev.bus, pciDev.tar.ByBits.dno );

    printf( "00/ %08lX/ DID = %04X, VID = %04X\n", ptr[0], pciHdr.DID, pciHdr.VID );
    printf( "04/ %08lX/ Status = %04X, Command = %04X\n", ptr[1], pciHdr.PCISTS, pciHdr.PCICMD );
    printf( "08/ %08lX/ Class = %02X %02X %02X, RevID = %02X\n", ptr[2], pciHdr.CLCD.base, pciHdr.CLCD.sub, pciHdr.CLCD.intf, pciHdr.RID );
    printf( "0C/ %08lX/ BIST = %02X, Hdr = %02X, Latency = %02X, Cashe Line Size = %02X\n", ptr[3], pciHdr.BIST, pciHdr.HDR, pciHdr.LAT, pciHdr.CALN );
    printf( "10/ %08lX/ ADDR0 = %08lX\n", ptr[4], pciHdr.Hdr.Type00h.BADR[0] );
    printf( "14/ %08lX/ ADDR1 = %08lX\n", ptr[5], pciHdr.Hdr.Type00h.BADR[1] );
    printf( "18/ %08lX/ ADDR2 = %08lX\n", ptr[6], pciHdr.Hdr.Type00h.BADR[2] );
    printf( "1C/ %08lX/ ADDR3 = %08lX\n", ptr[7], pciHdr.Hdr.Type00h.BADR[3] );
    printf( "20/ %08lX/ ADDR4 = %08lX\n", ptr[8], pciHdr.Hdr.Type00h.BADR[4] );
    printf( "24/ %08lX/ ADDR5 = %08lX\n", ptr[9], pciHdr.Hdr.Type00h.BADR[5] );
    printf( "28/ %08lX/ CIS Pointer= %08lX\n", ptr[10], pciHdr.Hdr.Type00h.CISP );
    printf( "2C/ %08lX/ SubsystemID = %04X, SubsystemVID = %04X\n", ptr[11], pciHdr.Hdr.Type00h.SID, pciHdr.Hdr.Type00h.SVID );
    printf( "30/ %08lX/ Expasion ROM addr = %08lX\n", ptr[12], pciHdr.Hdr.Type00h.EXROM );
    printf( "34/ %08lX/ New Capability Ptr = %02X\n", ptr[13], (USHORT)pciHdr.Hdr.Type00h.RESV[0]&0xFF );
    printf( "38/ %08lX/ reserved\n", ptr[14] );
    printf( "3C/ %08lX/ MaxLat = %02X, MinGnt = %02X, IntPin = %02X, IntLine = %02X\n", ptr[15], pciHdr.Hdr.Type00h.MAXLAT, pciHdr.Hdr.Type00h.MINGNT, pciHdr.Hdr.Type00h.INTPIN, pciHdr.Hdr.Type00h.INTLN );

	printf( "40/ %02X%02X%02X%02X/\n",
			pciHdr.vendorSpecific[0x03], pciHdr.vendorSpecific[0x02],
			pciHdr.vendorSpecific[0x01], pciHdr.vendorSpecific[0x00] );
	printf( "44/ %02X%02X%02X%02X/\n",
			pciHdr.vendorSpecific[0x07], pciHdr.vendorSpecific[0x06],
			pciHdr.vendorSpecific[0x05], pciHdr.vendorSpecific[0x04] );
	printf( "48/ %02X%02X%02X%02X/\n",
			pciHdr.vendorSpecific[0x0B], pciHdr.vendorSpecific[0x0A],
			pciHdr.vendorSpecific[0x09], pciHdr.vendorSpecific[0x08] );
	printf( "4C/ %02X%02X%02X%02X/\n",
			pciHdr.vendorSpecific[0x0F], pciHdr.vendorSpecific[0x0E],
			pciHdr.vendorSpecific[0x0D], pciHdr.vendorSpecific[0x0C] );
	printf( "50/ %02X%02X%02X%02X/\n",
			pciHdr.vendorSpecific[0x13], pciHdr.vendorSpecific[0x12],
			pciHdr.vendorSpecific[0x11], pciHdr.vendorSpecific[0x10] );
}

//=********************** EnumeratePciCard ****************
//=********************************************************
void		EnumeratePciCard(void)
{
	int         nbus=0, ndev=0, nfunc=0;
	int         ret;
	PCI_DEVICE	pciDev;
	PCI_HEADER	pciHdr;

	//
	//=== Find PCI Devices
	//

	printf( "bus:dev  DevID:VenID    memRTR     ioRTR    Space0(s0) Space1(s1) IRQ\n" );
	for( nbus=0; nbus<8; nbus++)
	for( ndev=0; ndev<32; ndev++)
	//for( nfunc=0; nfunc<8; nfunc++)
	{
		pciDev.bus = nbus;
		pciDev.tar.ByBits.fno = 0;
		pciDev.tar.ByBits.dno = ndev;
		ret = UniDrv_PCI_ReadCfgSpace( handle, &pciDev, &pciHdr );
		if( ret != 0)
			continue;
		if( (pciHdr.VID==0) || (pciHdr.VID==0xFFFF) )
			continue;
		printf("%02d:0x%02x 0x%04x:0x%04x 0x%08lx 0x%08lx 0x%08lx 0x%08lx %3d\n",
				nbus, ndev,
				pciHdr.DID, 
				pciHdr.VID,
				pciHdr.Hdr.Type00h.BADR[0],
				pciHdr.Hdr.Type00h.BADR[1],
				pciHdr.Hdr.Type00h.BADR[2],
				pciHdr.Hdr.Type00h.BADR[3],
				(USHORT)pciHdr.Hdr.Type00h.INTLN
				);
	}
}

//
// End of File
//
